#!/bin/tcsh

# script:  demo_suma_spheres.tcsh
# ver   :  1.0
# date  :  Feb 7, 2019
# author:  DR Glen (NIMH, SSCC)
#
# Several methods for showing spheres in SUMA.
#
# ======================================================================

# -------------------- Defining initial files --------------------------

# Start with a list of center coords and a chosen dset space
set sphere_coords = sphere_coords.1D
set templ         = MNI152_T1_2009c+tlrc
set templ_full    = `@FindAfniDsetPath "${templ}"`/${templ}

# Create a set of colors from SUMA's colormap.  This one was created
# by clicking the 'w' key over the ROI_256 color map color bar.
set sphere_colors = ROI_i256.1D.cmap

# -------------------- Preliminary steps ------------------------------

# Count how many spheres
set nsphere = `1dcat ${sphere_coords} | wc -l`
set lastrow = `ccalc -int -expr "${nsphere}-1"`

# Make some labels for each sphere.  Here, just a number (~counting or
# indexing).
count -digits 3 -col 1 ${nsphere} > vol_sphere_labels.1D

# Combine the labels with the coordinates
1dcat ${sphere_coords} vol_sphere_labels.1D > vol_sphere_coords_labeled.1D

# -------------------- Make volumes -----------------------------------

# Create spheres in a volume (from which we will make surfaces)
3dUndump                                  \
    -master ${templ_full}                 \
    -srad 3.5                             \
    -prefix vol_sphere                    \
    -datum byte                           \
    -overwrite                            \
    -xyz                                  \
    -orient RAI                           \
    vol_sphere_coords_labeled.1D

# This volume can be shown directly in suma or in the afni GUI and in
# the afni GUI's render plugin
cat << EOF

++ -----------------------------------------------------------------

   0) View the volume (as slices) in SUMA with:

        suma -vol vol_sphere+tlrc

    ... where you might want to hit ctrl+s to open the object
    controller, and then select the square by 'v' in 'Volume Rendering
    Controls' to view the spheres volumetrically.

    Also, you can include the reference volume for background
    purposes, via:

        suma -vol vol_sphere+tlrc \
            -vol ${templ_full} 

++ -----------------------------------------------------------------

EOF

# ----------------------- Surfaceize the volumes ------------------------

# Make surfaces (removing any ones that might exist from a previous
# run)
IsoSurface                               \
    -overwrite                           \
    -isorois                             \
    -mergerois+dsets                     \
    -o surf_sphere.gii                   \
    -input vol_sphere+tlrc               \
    -Tsmooth 0.3 60

cat << EOF

++ -----------------------------------------------------------------

   1) View the surfaces in SUMA

        suma -onestate -i surf_sphere*.gii

    or

        suma -onestate -i surf_sphere*.gii \
            -vol ${templ_full} 

++ -----------------------------------------------------------------

EOF

# ------------------- Icosahedrons ----------------------------

# Pre-clean (e.g., if re-running demo).
set ccc = `\ls ico_*`
if ( "$#ccc" != "0" ) then
    \rm ${ccc}
endif

# Make icosahedrons
foreach icoi ( `count -digits 3 1 ${nsphere}` )
    set xyz = `head -$icoi ${sphere_coords} | tail -1`

    CreateIcosahedron                       \
        -overwrite                          \
        -tosphere                           \
        -ctr $xyz                           \
        -prefix ico_${icoi}.gii             \
        -rad 3.5                            \
        -ld 20

    if ( "$icoi" == "001" ) then  
        SurfaceMetrics -i ico_${icoi}.gii -coords
    endif
    
    1deval                                  \
        -a ico_001.gii.coord.1D.dset        \
        -expr "$icoi" > ico_${icoi}.1D
end

# Combine the icosahedrons together in one surface file.
ConvertSurface                              \
    -overwrite                              \
    -onestate                               \
    -anatomical                             \
    -merge_surfs                            \
    -i ico_0??.gii                          \
    -o ico_combined.gii

cat ico_???.1D > ico_combined.1D

# The niml dset should be a little easier to work with, because
# it gets loaded automatically.
ConvertDset                                  \
    -overwrite                               \
    -i ico_combined.1D                       \
    -o ico_combined.niml.dset

#\rm -f ico_*.spec

cat << EOF

++ -----------------------------------------------------------------

   2) View the icosahedrons in SUMA

        suma -i ico_combined.gii

    or

        suma -i ico_combined.gii \
            -vol ${templ_full} 

    Pro-tips:
    + open the object controller (ctrl+s)
    + then color with ROI_256 colormap (selected with 'Cmp' button)
    + turn off 'sym'
    + set 'Min' to be 0 and 'Max' to be 255

++ -----------------------------------------------------------------

EOF

# ------------------ NIML Displayable Objects (Nidos) --------------------

# Pre-clean (e.g., if re-running demo).
set ccc = `\ls nido_*`
if ( "$#ccc" != "0" ) then
    \rm ${ccc}
endif

# Nidos are displayable and removable in suma, but they are not
# selectable, so you can't interact with these objects.
set sphere_do = nido_sphere.1D.do

# This is a technical label needed for the top of the file.
echo "#spheres" > $sphere_do

# Need unused alpha column for color.
1deval -a ${sphere_coords}'[0]' -expr '1' > nido_alpha.1D

# Need radius column.
1deval -a ${sphere_coords}'[0]' -expr '3.5' > nido_radius.1D

# Also, need mesh points filled code column.
1deval -a ${sphere_coords}'[0]' -expr '2' > nido_filled.1D

1dcat                                                 \
    ${sphere_coords} ${sphere_colors}"{0..$lastrow}"  \
    nido_alpha.1D                                     \
    nido_radius.1D                                    \
    nido_filled.1D  >> $sphere_do


cat << EOF

++ -----------------------------------------------------------------

   3) View the Nidos in SUMA, first loading up a background volume,
      and then instructing suma to load the spheres (the 'sleep'
      is for stability):

        suma -niml -vol ${templ_full}  &
        sleep 2
        DriveSuma                                          \
            -echo_edu                                      \
            -com viewer_cont -load_do $sphere_do

++ -----------------------------------------------------------------

EOF

# ------------------------ graph dataset -----------------------------

# Pre-clean (e.g., if re-running demo).
set ccc = `\ls graph_*`
if ( "$#ccc" != "0" ) then
    \rm ${ccc}
endif

# Convert the connection-coordinate info to a niml graph dataset

# Need unused alpha column for color.
1deval -a ${sphere_coords}'[0]' -expr '1' > graph_alpha.1D

1deval                                       \
    -a ${sphere_coords}'[0]'                 \
    -expr 'l'                                \
    > graph_sphere_index.1D

1dcat                                                    \
    graph_sphere_index.1D                                \
    graph_sphere_index.1D                                \
    graph_sphere_index.1D                                \
    ${sphere_colors}"{0..$lastrow}"                      \
    > graph_name_index.1D

# Make a faux edge here.
set lastrow = `ccalc -int -expr "${nsphere}-1"`

# Start the 1D file with some technical terminology.
echo "#edge  edgenode1 edgenode2" > graph_sphere_edge.1D

# Fill the file with some 'edge' connections
foreach isph (`count -digits 3 0 ${lastrow}`)
   set next_sph = `ccalc -int -expr "${isph}+1"`
   echo "1 $isph ${next_sph}" >> graph_sphere_edge.1D
end

#\rm ball_graph.niml.dset

ConvertDset                                                         \
    -overwrite                                                      \
    -o_niml_asc                                                     \
    -i       graph_alpha.1D                                         \
    -prefix  graph_set_sphere                                       \
    -add_node_index                                                 \
    -graph_named_nodelist_txt graph_name_index.1D ${sphere_coords}  \
    -graphize                                                       \
    -graph_edgelist_1D graph_sphere_edge.1D

cat << EOF

++ -----------------------------------------------------------------

   4) View graph dataset in suma with:

        suma -gdset graph_set_sphere.niml.dset

     or

        suma -gdset graph_set_sphere.niml.dset \
            -vol ${templ_full} 

++ -----------------------------------------------------------------

EOF
